// Additionnal vertex displacement
shared float4	PositionOffset;

// Diffuse map
texture			DiffuseMap0;
texture			DiffuseMap1;
texture			DiffuseMap2;

float4			Color0;
float4			Color1;
float4			Color2;



//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------



// Standard vertex shader input struct
struct	VS_Input
{
	float4	Position	:	POSITION;
	float2	DiffuseUv	:	TEXCOORD0;	
};


// Standard vertex shader output struct / pixel shader input struct
struct	VS_Output
{
	float4	Position	:	POSITION;
	float2	DiffuseUv	:	TEXCOORD0;	
};


// Standard vertex shader
VS_Output VS_Standard (const VS_Input input)
{
	VS_Output output;
	
	output.Position = input.Position + PositionOffset;
	output.DiffuseUv = 	input.DiffuseUv;
	
	return output;
}





// Multi-textured vertex shader input struct
struct	VS_M_Input
{
	float4	Position	:	POSITION;
	float2	DiffuseUv0	:	TEXCOORD0;
	float2	DiffuseUv1	:	TEXCOORD1;
	float2	DiffuseUv2	:	TEXCOORD2;
};


// Multi-textured vertex shader output struct / pixel shader input struct
struct	VS_M_Output
{
	float4	Position	:	POSITION;
	float2	DiffuseUv0	:	TEXCOORD0;	
	float2	DiffuseUv1	:	TEXCOORD1;	
	float2	DiffuseUv2	:	TEXCOORD2;	
};


// Multi-textured vertex shader
VS_M_Output	VS_Multitextured (const VS_M_Input input)
{
	VS_M_Output output;

	output.Position = input.Position + PositionOffset;
	output.DiffuseUv0 = 	input.DiffuseUv0;
	output.DiffuseUv1 = 	input.DiffuseUv1;
	output.DiffuseUv2 = 	input.DiffuseUv2;

	return output;
}



//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------




// Color only with alpha
technique	colorOnly
<
	int Priority = 1;
	int TechniqueIndex = 0;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		// no fog
		FogEnable = false;
		
		// vertex clipping & pixel clipping stage states
		CullMode			= NONE;
		ZEnable				= false;
		ZWriteEnable		= false;

		// alpha blending stage states
		AlphaTestEnable		= false;
		AlphaBlendEnable	= true;
		SrcBlend							= SRCALPHA;
		DestBlend						= INVSRCALPHA;

		// standard vertex shader
		VertexShader = compile vs_1_1 VS_Standard();

		// get color parameter in c0
		PixelShaderConstant[0] = <Color0>;

		PixelShader = 
		asm
		{
			ps_1_1
			
			// directly copy color to frame buffer
			mov r0, c0
		};
	}
}



// Diffuse map, no color, no alpha
technique	diffuseMap
<
	int Priority = 1;
	int TechniqueIndex = 1;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		// no fog
		FogEnable = false;
		
		// texture sampling stage states
		Texture[0]					= <DiffuseMap0>;
		MinFilter[0]					= LINEAR;
		MagFilter[0]				= LINEAR;
		MipFilter[0]					= LINEAR;
		AddressU[0]				= WRAP;
		AddressV[0]				= WRAP;

		// vertex clipping & pixel clipping stage states
		CullMode						= NONE;
		ZEnable							= false;
		ZWriteEnable				= false;

		// alpha blending stage states
		AlphaTestEnable		= false;
		AlphaBlendEnable	= true;
		SrcBlend							= SRCALPHA;
		DestBlend						= INVSRCALPHA;

		// standard vertex shader
		VertexShader = compile vs_1_1 VS_Standard();

		// pixel shader
		PixelShader = 
		asm
		{
			ps_1_1

			def c1, 0.5f,0.0f,0.0f,0.5f

			// convert tex coord 0 to sampled texel in TPU 0
			tex	t0

			// simply copy texel to frame buffer
			mov r0, t0

			// copy texel and add a transparent red
			// add r0.rgb, c1, t0 + mov r0.a, c1.a
		};
	}
}



// Colored diffuse map with alpha
technique	coloredDiffuseMap
<
	int Priority = 1;
	int TechniqueIndex = 2;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		// no fog
		FogEnable = false;
		
		// texture sampling stage states
		Texture[0]					= <DiffuseMap0>;
		MinFilter[0]					= LINEAR;
		MagFilter[0]				= LINEAR;
		MipFilter[0]					= LINEAR;
		AddressU[0]				= WRAP;
		AddressV[0]				= WRAP;

		// vertex clipping & pixel clipping stage states
		CullMode						= NONE;
		ZEnable							= false;
		ZWriteEnable				= false;

		// alpha blending stage states
		AlphaTestEnable		= false;
		AlphaBlendEnable	= true;
		SrcBlend							= SRCALPHA;
		DestBlend						= INVSRCALPHA;

		// standard vertex shader
		VertexShader = compile vs_1_1 VS_Standard();

		// get color parameter in c0
		PixelShaderConstant[0] = <Color0>;

		// pixel shader
		PixelShader = 
		asm
		{
			ps_1_1

			// convert tex coord 0 to sampled texel in TPU 0
			tex	t0

			// copy modulated texel with color to frame buffer
			mul r0, t0.rgba, c0.rgba
		};
	}
}



// Multi textured ( 3 layers ) colored diffuse map with alpha
technique multiTexturedColoredDiffuseMap
<
	int Priority = 1;
	int TechniqueIndex = 3;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		// no fog
		FogEnable = false;
		
		// texture sampling stage states (TPU 0)
		Texture[0]				= <DiffuseMap0>;
		MinFilter[0]				= LINEAR;
		MagFilter[0]			= LINEAR;
		MipFilter[0]				= LINEAR;
		AddressU[0]			= WRAP;
		AddressV[0]			= WRAP;

		// texture sampling stage states (TPU 1)
		Texture[1]				= <DiffuseMap1>;
		MinFilter[1]				= LINEAR;
		MagFilter[1]			= LINEAR;
		MipFilter[1]				= LINEAR;
		AddressU[1]			= WRAP;
		AddressV[1]			= WRAP;

		// texture sampling stage states (TPU 2)
		Texture[2]				= <DiffuseMap2>;
		MinFilter[2]				= LINEAR;
		MagFilter[2]			= LINEAR;
		MipFilter[2]				= LINEAR;
		AddressU[2]			= WRAP;
		AddressV[2]			= WRAP;

		// vertex clipping & pixel clipping stage states
		CullMode					= NONE;
		ZEnable					= false;
		ZWriteEnable			= false;

		// alpha blending stage states
		AlphaTestEnable		= false;
		AlphaBlendEnable	= true;
		SrcBlend					= SRCALPHA;
		DestBlend				= INVSRCALPHA;

		// multi textured vertex shader
		VertexShader = compile vs_1_1 VS_Multitextured();

		// get 3 color parameters in c0, c1, c2
		PixelShaderConstant[0] = <Color0>;
		PixelShaderConstant[1] = <Color1>;
		PixelShaderConstant[2] = <Color2>;

		// vertex shader
		PixelShader = 
		asm
		{
			ps_1_1

			tex	t0
			tex	t1
			tex	t2

			mul r0, t0, c0.a																					// modulate texel by alpha
			mul r0.rgb, r0, c0 + mul r0.a, t0.a, c0.a								// modulate by color and alpha

			mul r1, t1, c1.a																					// modulate texel by alpha
			mad r0.rgb, r1, c1, r0 + mad r0.a, t1.a, c1.a, r0.a		// modulate by color and alpha

			mul r1, t2, c2.a																					// modulate texel by alpha
			mad r0.rgb, r1, c2, r0 + mad r0.a, t2.a, c2.a, r0.a	 	// modulate by color and alpha
			
//			mul r0, t0, c0.a																		// modulate texel by alpha
//			mul r0.rgb, r0, c0 + mov r0.a, c0.a							// modulate by color and alpha
//
//			mul r1, t1, c1.a																		// modulate texel by alpha
//			mad r0.rgb, r1, c1, r0 + add r0.a, r0.a, c1.a		// modulate by color and alpha
//
//			mul r1, t2, c2.a																		// modulate texel by alpha
//			mad r0.rgb, r1, c2, r0 + add r0.a, r0.a, c2.a		// modulate by color and alpha
		};
	}
}