//------------------------------------------------------------------------------------------------------
// Mini map shader (compass, with HW rotation, zoom, translate)
//
//------------------------------------------------------------------------------------------------------

#include "shared.fx"

//------------------------------------------------------------------------------------------------------
//- Static parameters
//------------------------------------------------------------------------------------------------------

texture			MapBackGround;
texture			MapBorder;

float4			ViewerPos;
float			ViewerYaw;
float			ZoomFactor;

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

struct	GUI_MAP_VS_INPUT
{
	float4	Position	:	POSITION;	
	float2	DiffuseUV	:	TEXCOORD0;	
};


struct	GUI_MAP_SIMPLE_VS_INPUT
{
	float4	Position		:	POSITION;
	float2	MapBackGroundUV	:	TEXCOORD0;
	float2	MapBorderUV		:	TEXCOORD1;
	
};

//------------------------------------------------------------------------------------------------------

struct	GUI_MAP_VS_OUTPUT
{
	float4	Position		:	POSITION;
	float2	MapBackGroundUV	:	TEXCOORD0;
	float2	MapBorderUV		:	TEXCOORD1;
};


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

GUI_MAP_VS_OUTPUT SimpleVS(const GUI_MAP_SIMPLE_VS_INPUT input)
{
	GUI_MAP_VS_OUTPUT output;
	output.Position = mul (input.Position, WorldCameraProjection);
	output.MapBackGroundUV = input.MapBackGroundUV;
	output.MapBorderUV = input.MapBorderUV;
	
	return output;
}

GUI_MAP_VS_OUTPUT MiniMapVS(const GUI_MAP_VS_INPUT input)
{
	GUI_MAP_VS_OUTPUT output;	
	
	//Position --------------------------------------------------------------------
	// output position into world+view+projection space
	output.Position = mul (input.Position, WorldCameraProjection);
	
	// UV Radar Map calculations --------------------------------------------------			  
	// offset center of rotation to match center of radar (as we are about to rotate around 0,0...)
	float offsetedX = input.DiffuseUV.x - 0.5f; // the mapping we used on the quad is full map uv (0,0) to (1,1) so (0.5,0.5) is the right remap
	float offsetedY = input.DiffuseUV.y - 0.5f; // as the rotation will occur around the origin, we just displace the center of the quad as the zero UV.
	float cYaw = cos(ViewerYaw);
	float sYaw = sin(ViewerYaw);

	output.MapBackGroundUV.x = (offsetedX * cYaw + offsetedY * sYaw);  // rotate around origin
	output.MapBackGroundUV.y = (offsetedX * -sYaw + offsetedY * cYaw); // rotate around origin

	// and the map offset player position and zoom on the map UV
	output.MapBackGroundUV.x = (output.MapBackGroundUV.x/ZoomFactor) + ViewerPos.x;
	output.MapBackGroundUV.y = (output.MapBackGroundUV.y/ZoomFactor) + ViewerPos.y;

	// UV Radar Edge transfer -----------------------------------------------------			     	   		
	output.MapBorderUV = input.DiffuseUV;
   	
	return output;
}


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
technique	GuiMapTech0
<
	int Priority			= 0;
	int TechniqueIndex		= 0;
	int NeedSorting			= 1;
	int DeviceType			= HWSHADER_ONLY;
	int LightingType		= INTEGRATED_LIGHTING;
	string RenderingType	= "Standard";
>
{
	pass pass1
	{
		FogEnable = FALSE;
	
		Texture[0]		= <MapBackGround>;
		MinFilter[0]	= LINEAR; //trilinear filtering
		MagFilter[0]	= LINEAR;
		MipFilter[0]	= LINEAR;
		AddressU[0]		= CLAMP;
		AddressV[0]		= CLAMP;
				
		Texture[1]		= <MapBorder>;
		MinFilter[1]	= LINEAR; //trilinear filtering
		MagFilter[1]	= LINEAR;
		MipFilter[1]	= LINEAR;
		AddressU[1]		= CLAMP;
		AddressV[1]		= CLAMP;		

		CullMode		= CW;
		ZEnable			= true;
		ZFunc			= LESSEQUAL;
		ZWriteEnable	= false;
		AlphaBlendEnable= true; 
		SrcBlend		= SRCALPHA;
		DestBlend		= INVSRCALPHA;	
		FogEnable		= false;		   
		
		AlphaTestEnable	= false;		
			
		//VertexShader = compile vs_1_1 MiniMapVS();
		VertexShader = compile vs_1_1 SimpleVS();

		// t0 -> Diffuse Map : rotating minimap
		// t1 -> Diffuse	 : compass border and alpha
				
		PixelShader = 
		asm
		{
			ps_1_1
						
			tex	t0
			tex t1
		   
			mad r0, t0, t1.a, t1
		};
	}
}

//------------------------------------------------------------------------------------------------------

#include "mesh_shadow.fx"
#include "mesh_shadow_projector.fx"

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

