//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#include "shared.fx"
#include "lighting.fx"
#define FOG_DISABLE
#include "fog.fx"

//------------------------------------------------------------------------------------------------------
//- Static parameters
//------------------------------------------------------------------------------------------------------

texture			DiffuseMap;
float4			ColorAndAlpha; //shared between all occurences of this shader.

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

struct	MeshAlphaD1AlphaVertexShaderInput
{
	float4	Position	:	POSITION;
	float2	DiffuseUv	:	TEXCOORD0;	
};

//------------------------------------------------------------------------------------------------------

struct	MeshAlphaD1AlphaVertexShaderOutput
{
	float4	Position		:	POSITION;
	float4	ColorAndAlpha	:	COLOR0;
	float2	DiffuseUv		:	TEXCOORD0;
	FOG_OPTION_VERTEX_FIELD
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

MeshAlphaD1AlphaVertexShaderOutput	MeshAlphaD1AlphaVertexShaderTech0Pass1(const MeshAlphaD1AlphaVertexShaderInput input);

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

MeshAlphaD1AlphaVertexShaderOutput	MeshAlphaD1AlphaVertexShaderTech0Pass1(const MeshAlphaD1AlphaVertexShaderInput input)
{
	MeshAlphaD1AlphaVertexShaderOutput output;
	
	// output position into world+view+projection space
	output.Position = mul (input.Position,WorldCameraProjection);
		
	// Diffuse Uv output
	output.DiffuseUv = input.DiffuseUv;
	
	// use color0 to stock the fade factor!!
	output.ColorAndAlpha = ColorAndAlpha;
	
	// fog computation
	FOG_OPTION_COMPUTE(output, output.Position);
	
	return output;
}


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
technique	TextureOnly
<
	int Priority = 0;
	int NeedSorting	= 1;
	int TechniqueIndex = 0;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		Texture[0]			= <DiffuseMap>;
		MinFilter[0]		= LINEAR;
		MagFilter[0]		= LINEAR;
		MipFilter[0]		= LINEAR;
		AddressU[0]			= WRAP;
        AddressV[0]			= WRAP;
		CullMode			= CW;
		ZEnable				= true;
		ZFunc				= LESSEQUAL;
		ZWriteEnable		= true;
		
		AlphaTestEnable		= false;
		AlphaBlendEnable	= true;		//use alpha blend !!! I beg thee.
		SrcBlend			= SRCALPHA;
		DestBlend			= INVSRCALPHA;
		FOG_OPTION_PARAMETERS;	
		
		
		VertexShader = compile vs_1_1 MeshAlphaD1AlphaVertexShaderTech0Pass1();
		
		PixelShader = 
		asm
		{
			ps_1_1
						 
			tex		t0	// Diffuse map								
			
			mov r0.rgb,t0 + mul r0.a, t0.a, v0.a // ponderate the texture alpha by a global alpha blend -> fade. 			
		};
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
technique	ColorOnly
<
	int Priority = 0;
	int NeedSorting	= 1;
	int TechniqueIndex = 1;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		Texture[0]			= <DiffuseMap>;
		MinFilter[0]		= LINEAR;
		MagFilter[0]		= LINEAR;
		MipFilter[0]		= LINEAR;
		AddressU[0]			= WRAP;
        AddressV[0]			= WRAP;
		CullMode			= CW;
		ZEnable				= true;
		ZFunc				= LESSEQUAL;
		ZWriteEnable		= true;
		
		AlphaTestEnable		= false;
		AlphaBlendEnable	= true;		//use alpha blend !!! I beg thee.
		SrcBlend			= SRCALPHA;
		DestBlend			= INVSRCALPHA;
		FOG_OPTION_PARAMETERS;	
		
		
		VertexShader = compile vs_1_1 MeshAlphaD1AlphaVertexShaderTech0Pass1();
		
		PixelShader = 
		asm
		{
			ps_1_1
						 
			tex		t0	// Diffuse map					
			
			mul r0.rgb,t0,v0 + mul r0.a, t0.a, v0.a // ponderate the texture alpha by a global alpha blend -> fade. 						
			
		};
	}
}


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
technique	techTnL_0
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = TNL_ONLY;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		WorldTransform[0]	= <WorldCameraProjection>;
		
		Texture[0]			= <DiffuseMap>;
		
		ColorArg1[0]		= TEXTURE;
		ColorOp[0]			= SELECTARG1;
		AlphaArg1[0]		= TEXTURE;
		AlphaOp[0]			= SELECTARG1;
		
		ColorOp[1]			= DISABLE;
		AlphaOp[1]			= DISABLE;
		
		MinFilter[0]		= LINEAR;
		MagFilter[0]		= LINEAR;
		MipFilter[0]		= LINEAR;
		
		AddressU[0]			= WRAP;
        AddressV[0]			= WRAP;
        
		CullMode			= CW;
		ZEnable				= true;
		ZFunc				= LESSEQUAL;
		ZWriteEnable		= true;
		AlphaBlendEnable	= false;
		AlphaTestEnable		= true;
		AlphaRef			= 192;
		AlphaFunc			= GREATEREQUAL;
		
		VertexShader 		= NULL;		
		PixelShader 		= NULL;
	}
}
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#include "mesh_shadow.fx"
#include "mesh_shadow_projector.fx"

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

