//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#include "shared.fx"
#include "lighting.fx"
#define FOG_DISABLE
#include "fog.fx"


//------------------------------------------------------------------------------------------------------
//- Static parameters
//------------------------------------------------------------------------------------------------------

texture			DiffuseMap;

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

struct	CVertexShaderInput
{
	float4	Position	:	POSITION;
	float2	DiffuseUv	:	TEXCOORD0;	
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderOutput
{
	float4	Position			:	POSITION;
	float2	DiffuseUv			:	TEXCOORD0;
	FOG_OPTION_VERTEX_FIELD_HACK(COLOR0)
	//FOG_OPTION_VERTEX_FIELD
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderOutput	OpaSelfIllumGeomTech1Pass1			(const CVertexShaderInput input);

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderOutput	OpaSelfIllumGeomTech1Pass1 (const CVertexShaderInput input)
{
	CVertexShaderOutput output;
	
	// output position into world+view+projection space
	output.Position = mul (input.Position,WorldCameraProjection);
		
	// Diffuse Uv output
	output.DiffuseUv = input.DiffuseUv;
	
	// fog computation
	FOG_OPTION_ON(float rFog = fog_linear(output.Position); output.Hack = float4(rFog, rFog, rFog, rFog);)	

	return output;
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#define RenderStateBlock \
Texture[0]			= <DiffuseMap>; \
MinFilter[0]		= LINEAR; \
MagFilter[0]		= LINEAR; \
MipFilter[0]		= LINEAR; \
AddressU[0]			= WRAP; \
AddressV[0]			= WRAP; \
CullMode			= CW; \
ZEnable				= true; \
ZFunc				= LESSEQUAL; \
ZWriteEnable		= false; \
AlphaBlendEnable	= true; \
SrcBlend			= ONE; \
DestBlend			= ONE; \
AlphaTestEnable		= false; \
FogEnable			= false \
//FOG_OPTION_PARAMETERS


technique	tech1
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = HWSHADER_ONLY;
	int	NeedSorting = 1;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		
		RenderStateBlock;

		VertexShader = compile vs_1_1 OpaSelfIllumGeomTech1Pass1();
		
		PixelShader = 
		#ifdef FOG_OPTION_ENABLE		
			asm
			{
				ps_1_1
				
				tex		t0	// Diffuse map
				
				//mov r0,t0
				mul_sat r0, t0, v0	
			};
		#else
			asm
			{
				ps_1_1
				
				tex		t0	// Diffuse map
				
				mov r0,t0			
			};
		#endif
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	techTnL_0
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = TNL_ONLY;
	int	NeedSorting = 1;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		WorldTransform[0]	= <WorldCameraProjection>;
		IndexedVertexBlendEnable = false;
		
		Texture[0]			= <DiffuseMap>;
		MinFilter[0]		= LINEAR;
		MagFilter[0]		= LINEAR;
		MipFilter[0]		= LINEAR;
		AddressU[0]			= WRAP;
        AddressV[0]			= WRAP;
		CullMode			= CW;
		ZEnable				= true;
		ZFunc				= LESSEQUAL;
		ZWriteEnable		= false;
		AlphaBlendEnable	= true;
		SrcBlend			= ONE;
		DestBlend			= ONE;
		AlphaTestEnable		= false;
		
		ColorArg1[0]	= TEXTURE;
		ColorOp[0]		= SELECTARG1;
		AlphaArg1[0]	= TEXTURE;
		AlphaOp[0]		= SELECTARG1;
		
		ColorOp[1]		= DISABLE;
		AlphaOp[1]		= DISABLE;
		
		VertexShader 		= NULL;		
		PixelShader 		= NULL;
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#include "mesh_shadow.fx"
#include "mesh_shadow_projector.fx"

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------