//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#include "shared.fx"
#include "lighting.fx"
#include "fog.fx"

//------------------------------------------------------------------------------------------------------
//- Static parameters
//------------------------------------------------------------------------------------------------------

texture			DiffuseMap;
texture			NormalMap;
texture			GlossinessMap;
texture			SpecularMap;
texture			LightMap;


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

struct	CVertexShaderInput
{
	float4	Position	:	POSITION;
	float3	Normal		:	NORMAL;	
	float2	DiffuseUv	:	TEXCOORD0;	
	float2  LightUv		:  	TEXCOORD1;
	float3	BumpTangent	:	TEXCOORD2;	
	float3	BumpNormal	:	TEXCOORD3;
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderOutput
{
	float4	Position			:	POSITION;
	float2	DiffuseUv			:	TEXCOORD0;	
	float2	LightUv				:	TEXCOORD1;
	FOG_OPTION_VERTEX_FIELD
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderOutputLighting
{
	float4	Position			:	POSITION;
	float2	DiffuseUv			:	TEXCOORD0;	
	float2	NormalUv			:	TEXCOORD1;
	float2	LightmapUv			:	TEXCOORD2;
	FOG_OPTION_ON(float4	LightVector :	COLOR0;) // store fog in v0.a
	FOG_OPTION_OFF(	float3	LightVector :	COLOR0;)
	float3	DiffuseColor		:	COLOR1;
//	FOG_OPTION_VERTEX_FIELD_HACK(TEXCOORD3)
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderOutputSpec
{
	float4	Position					:	POSITION;
	float2	DiffuseUv					:	TEXCOORD0;	
	float2	NormalUv					:	TEXCOORD1;
	float3	TextureSpaceLightVector		:	TEXCOORD2;	
	float3	TextureSpaceLightVectorHalf	:	TEXCOORD3;
	FOG_OPTION_ON(float4 SpecularStrength:	COLOR0;) // store fog in v0.a	
	FOG_OPTION_OFF(float3 SpecularStrength:	COLOR0;)
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderOutput OpaLightmapGeomTech1Pass1(const CVertexShaderInput input);
CVertexShaderOutputLighting OpaLightmapGeomTech2Pass1(const CVertexShaderInput input);
CVertexShaderOutputSpec	OpaBumpLitGeomTech4Pass1 (const CVertexShaderInput input);

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderOutput	OpaLightmapGeomTech1Pass1 (const CVertexShaderInput input)
{
	CVertexShaderOutput output;
	
	// output position into world+view+projection space
	output.Position = mul (input.Position,WorldCameraProjection);
		
	// Diffuse Uv output
	output.DiffuseUv = input.DiffuseUv;
		
	// LightmapUv Uv output
	output.LightUv = input.LightUv;
			
	// fog computation
	FOG_OPTION_COMPUTE(output, output.Position);		
	
	return output;
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderOutputLighting	OpaLightmapGeomTech2Pass1 (const CVertexShaderInput input)
{
	CVertexShaderOutputLighting output;
	
	// output position into world+view+projection space
	output.Position = mul (input.Position,WorldCameraProjection);
		
	// Diffuse Uv output
	output.DiffuseUv = input.DiffuseUv;
	
	// Normal Uv output
	output.NormalUv = input.DiffuseUv;

	// LightmapUv Uv output
	output.LightmapUv = input.LightUv;
			
	// omnilight direction computation
	float4 omniDirection = ComputeVectorAndLength (ObjectLocalLightPosition[0],input.Position);
	
	// omnilight direction attenuation
	//omniDirection *= ComputeAttenuation (omniDirection.w,LightAttenuationFarStart[0],LightAttenuationFarEnd[0],LightAttenuationDelta[0]);
	
	// diffuse Color
	output.DiffuseColor = DiffuseColor[0].xyz;
//	output.DiffuseColor *= ComputeAttenuation (omniDirection.w,LightAttenuationFarStart[0],LightAttenuationFarEnd[0],LightAttenuationDelta[0]);

	// omnilight output to diffuse
	
	float3 omniDirectionTextureSpace;
	float3 binormal = input.Normal;
	
	// omnilight direction to texture space
	omniDirectionTextureSpace = TextureSpaceTransform (omniDirection.xyz, input.BumpTangent, input.BumpNormal, binormal);
	omniDirectionTextureSpace *= ComputeAttenuation (omniDirection.w, LightAttenuationFarStart[0], LightAttenuationFarEnd[0], LightAttenuationDelta[0]);
		
	// omnilight 1 output to diffuse
	output.LightVector.xyz = omniDirectionTextureSpace * 0.5f + 0.5f;
		
	// fog computation
	FOG_OPTION(output.LightVector.w = fog_linear(output.Position));
	
	return output;
}


CVertexShaderOutputSpec	OpaBumpLitGeomTech4Pass1 (const CVertexShaderInput input)
{
	CVertexShaderOutputSpec output;
	
	// output position into world+view+projection space
	output.Position = mul (input.Position,WorldCameraProjection);

	// binormal computation based on input tangent and normal in local space
	float3 binormal = input.Normal;
	
	// Camera vertex direction
	float3 cameraDirection = ComputeVector (ObjectLocalCameraPosition,input.Position);
	
	// Light vertex direction
	float4 omniDirection1 = ComputeVectorAndLength (ObjectLocalLightPosition[0],input.Position);
		
	// half view vector computation
	float3 halfView1 = ComputeHalfViewVector (omniDirection1.xyz,cameraDirection);
	
			
	//float3 halfView1 = reflect (-omniDirection1.xyz,cameraDirection);

	// omni direction to texture space
	float3 omniDirectionTextureSpace = TextureSpaceTransform (omniDirection1.xyz,input.BumpTangent,input.BumpNormal,binormal);
		
	output.TextureSpaceLightVector = omniDirectionTextureSpace * 0.5f + 0.5f;
			
	// half view vector to texture space
	float3 halfView1TextureSpace = TextureSpaceTransform (halfView1,input.BumpTangent,input.BumpNormal,binormal);
	
	// half view vector 1 output to diffuse
	output.TextureSpaceLightVectorHalf = halfView1TextureSpace * 0.5f + 0.5f;
		
	// Diffuse Uv output
	output.DiffuseUv = input.DiffuseUv;
	
	// Normal Uv output
	output.NormalUv = input.DiffuseUv;
	
	output.SpecularStrength = dot (halfView1TextureSpace,binormal);
	output.SpecularStrength *= ComputeAttenuation (omniDirection1.w,LightAttenuationFarStart[0],LightAttenuationFarEnd[0],LightAttenuationDelta[0]);
	
	// fog computation
	FOG_OPTION_ON(output.SpecularStrength.w = fog_linear(output.Position);)

	return output;
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

sampler DiffuseSampler = sampler_state
{
	Texture		= <DiffuseMap>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
    AddressV	= WRAP;
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

/*

sampler LightTextureSampler = sampler_state
{
	Texture		= <LightTexture>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
    AddressV	= CLAMP;
}; */

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------


sampler NormalSampler = sampler_state
{
	Texture		= <NormalMap>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
    AddressV	= WRAP;
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

sampler SpecularSampler = sampler_state
{
	Texture		= <SpecularMap>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
    AddressV	= WRAP;
};

sampler GlossinessSampler = sampler_state
{
	Texture		= <GlossinessMap>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
    AddressV	= CLAMP;
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

sampler LightMapSampler = sampler_state
{
	Texture		= <LightMap>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
    AddressV	= WRAP;
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#define Pass1RenderStateBlock \
CullMode			= CW; \
ZEnable				= true; \
ZFunc				= LESSEQUAL; \
ZWriteEnable		= true; \
AlphaBlendEnable	= false; \
AlphaTestEnable		= true; \
AlphaRef			= 192; \
AlphaFunc			= GREATEREQUAL ; \
FOG_OPTION_PARAMETERS

//------------------------------------------------------------------------------------------------------

#define LightingRenderStateBlock \
CullMode			= CW; \
ZEnable				= true; \
ZFunc				= LESSEQUAL; \
ZWriteEnable		= false; \
AlphaBlendEnable	= true; \
SrcBlend			= ONE; \
DestBlend			= ONE; \
AlphaTestEnable		= true; \
AlphaRef			= 0; \
AlphaFunc			= GREATEREQUAL ; \
FogEnable			= false


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	tech1
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = MULTILIGHTSET_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		Sampler[0]	= <DiffuseSampler>;
		Sampler[1]	= <LightMapSampler>;
		
		Pass1RenderStateBlock;
		
		VertexShader = compile vs_1_1 OpaLightmapGeomTech1Pass1();
		
		PixelShader = 
		asm
		{
			ps_1_1
			
			tex			t0	// Diffuse map
			tex			t1	// Light map
						
			mul_d2		r0.rgb,t1,t1.a + mov r0.a,t0.a // lightmap darker
			//mul		r0.rgb,t1,t1.a + mov r0.a,t0.a
			mul_x4_sat r0.rgb,r0,t0
		};
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	tech2
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = MULTILIGHTSET_LIGHTING;
	string RenderingType = "Lighting";
	int LightsetIndex = 0;
>
{
	pass pass1
	{
		Sampler[0]	= <DiffuseSampler>;
		Sampler[1]	= <NormalSampler>;
		Sampler[2]	= <LightMapSampler>;
		
		LightingRenderStateBlock;
		
		VertexShader = compile vs_1_1 OpaLightmapGeomTech2Pass1();
		
		PixelShader =
		#ifdef FOG_OPTION_ENABLE 
		asm
		{
			ps_1_1
						
			tex			t0	// Diffuse map
			tex			t1	// Normal map
			tex         t2  // Light map to control bump
			
			//mul		r1.rgb,t2,t2.a + mov r1.a,t0.a
			mul_d2		r1.rgb,t2,t2.a + mov r1.a,t0.a // darker
			mul_x4_sat r1.rgb,t0,r1
			
			// diffuse bump
			dp3_sat r0.rgb,t1_bx2,v0_bx2 + mov r0.a,t0.a
			mul r0.rgb,r0,v1
			mul_x2 r0.rgb,r0, r1
			
			// fog hack
			mul_sat r0.rgb, r0, v0.a
		};
		#else
		asm
		{
			ps_1_1
						
			tex			t0	// Diffuse map
			tex			t1	// Normal map
			tex         t2  // Light map to control bump
			
			//mul		r1.rgb,t2,t2.a + mov r1.a,t0.a
			mul_d2		r1.rgb,t2,t2.a + mov r1.a,t0.a // darker
			mul_x4_sat r1.rgb,t0,r1
			
			// diffuse bump
			dp3_sat r0.rgb,t1_bx2,v0_bx2 + mov r0.a,t0.a
			mul r0.rgb,r0,v1
			mul_x2 r0.rgb,r0, r1
		};
		#endif
	}
}


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	tech3
<
	int Priority = 1;
	int TechniqueIndex = 1;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = MULTILIGHTSET_LIGHTING;
	string RenderingType = "Lighting";
	int LightsetIndex = 1;
>
{
	pass pass1
	{
		Sampler[0]	= <DiffuseSampler>;
		Sampler[1]	= <NormalSampler>;
				
		LightingRenderStateBlock;
		
		VertexShader = compile vs_1_1 OpaLightmapGeomTech2Pass1();
		
		PixelShader =
		asm
		{
			ps_1_1
						
			tex			t0	// Diffuse map
			tex			t1	// Normal map
		
			// diffuse bump without lightmap
			dp3_sat r0.rgb,t1_bx2,v0_bx2 + mov r0.a,t0.a
		
			mul r0.rgb,r0,v1
			mul_x2 r0.rgb,r0, t0
			
			// fog hack
			mul_sat r0.rgb, r0, v0.a
		};
	}
 }

/////////////////////////////////////////////////////////////////////////////////////////////////////////////

technique SpecularPass // tech4
<
  int Priority = 1;
  int TechniqueIndex = 2;
  int DeviceType = HWSHADER_ONLY;
  int LightingType = MULTILIGHTSET_LIGHTING;
  string RenderingType = "Lighting";
  int IntegratedPostLighting = 1; /* NEW FLAG */
  int LightsetIndex = 2;
>
{
	pass pass1
	{
	
		Sampler[0]	= <SpecularSampler>;
		Sampler[1]	= <NormalSampler>;
		Sampler[3]	= <GlossinessSampler>;
        
        LightingRenderStateBlock;
		
		VertexShader = compile vs_1_1 OpaBumpLitGeomTech4Pass1();
		
		PixelShaderConstant[2]		= <SpecularColor>;
		
		PixelShader = 
		asm
		{
			ps_1_1
			
//			def c2, 0.08, 0.12, 0.08, 1.0 // no specular
			
			tex			t0	// Specular map
			tex			t1	// Normal map
			texm3x2pad	t2, t1_bx2 // Light vector * normal vector
			texm3x2tex	t3, t1_bx2 // Half vector * normal vector
			
			mov_sat r0.rgb,t3 + mov r0.a,t0.a
			mul r0.rgb,r0,v0
			mul r0.rgb,r0,t0
			mul_x4 r0.rgb,r0,c2
			
			// fog hack
			mul_sat r0.rgb, r0, v1.a
		};
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	techTnL_0
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = TNL_ONLY;
	int LightingType = ACCUMULATED_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		WorldTransform[0]	= <WorldCameraProjection>;
		
		Sampler[0]	= <LightMapSampler>;
		Sampler[1]	= <DiffuseSampler>;
		
		Pass1RenderStateBlock;
		
		TexCoordIndex[0]	= 1;
		TexCoordIndex[1]	= 0;
		TextureFactor		= {0.0f,0.0f,0.0f,0.0f};
		
		ColorArg1[0]	= TEXTURE;
		ColorArg2[0]	= TFACTOR;
		ColorOp[0]		= BLENDTEXTUREALPHA;
		AlphaArg1[0]	= TEXTURE;
		AlphaOp[0]		= SELECTARG1;
		
		ColorArg1[1]	= TEXTURE;
		ColorArg2[1]	= CURRENT;
		AlphaArg1[1]	= TEXTURE;
		AlphaOp[1]		= SELECTARG1;
		ColorOp[1]		= MODULATE4X;
		
		ColorOp[2]		= DISABLE;
		AlphaOp[2]		= DISABLE;
					
		VertexShader 		= NULL;		
		PixelShader 		= NULL;
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#include "mesh_shadow.fx"
#include "mesh_shadow_projector.fx"

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
