//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#include "shared.fx"
#include "lighting.fx"
#include "fog.fx"

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------
//- Static parameters
//------------------------------------------------------------------------------------------------------

texture			Diffuse1;
texture			Diffuse2;
texture			NormalMap;
texture			GlossinessMap;
texture			SpecularMap;

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

struct	CVertexShaderInput
{
	float4	Position	:	POSITION;
	float3	Normal		:	NORMAL;	
	float2	Diffuse1Uv	:	TEXCOORD0;
	float2	Diffuse2Uv	:	TEXCOORD1;
	float3	BumpTangent	:	TEXCOORD2;	
	float3	BumpNormal	:	TEXCOORD3;
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderOutput
{
	float4	Position					:	POSITION;
	float2	Diffuse1Uv					:	TEXCOORD0;	
	float2	Diffuse2Uv					:	TEXCOORD1;	
	float2	NormalUv					:	TEXCOORD2;
	float3	TextureSpaceLightVector1	:	COLOR0;	
	float3	TextureSpaceLightVector2	:	COLOR1;	
	FOG_OPTION_VERTEX_FIELD
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderOutputTech2
{
	float4	Position					:	POSITION;
	float2	Diffuse1Uv					:	TEXCOORD0;	
	float2	Diffuse2Uv					:	TEXCOORD1;	
	float3	DiffuseColor				:	COLOR0;	
	FOG_OPTION_VERTEX_FIELD
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderOutputSpec
{
	float4	Position					:	POSITION;
	float2	SpecularUv					:	TEXCOORD0;
	float2	NormalUv					:	TEXCOORD1;
	float3	TextureSpaceLightVector		:	TEXCOORD2;	
	float3	TextureSpaceLightVectorHalf	:	TEXCOORD3;
	FOG_OPTION_ON(float4 SpecularStrength:	COLOR0;) // store fog in v0.a
	FOG_OPTION_OFF(float3 SpecularStrength:	COLOR0;)
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

// ambient + 2 omnis + bump + specular (2 passes : diffuse + specular)

CVertexShaderOutput			OpaBumpLitGeomTech1Pass1	(const CVertexShaderInput input);
CVertexShaderOutputSpec		OpaBumpLitGeomTech1Pass2	(const CVertexShaderInput input);
CVertexShaderOutputTech2	OpaBumpLitGeomTech2			(const CVertexShaderInput input);

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderOutput	OpaBumpLitGeomTech1Pass1 (const CVertexShaderInput input)
{
	CVertexShaderOutput output;
	
	// output position into world+view+projection space

	output.Position = mul (input.Position,WorldCameraProjection);
	
	// binormal computation based on input tangent and normal in local space	
	//float3 binormal = ComputeBinormal (input.BumpTangent,input.BumpNormal,input.Normal);
	float3 binormal = input.Normal;

	float3 omniDirectionTextureSpace[2];
	
	for (int i=0;i<2;i++)
	{
		// omnilight direction computation
		float4 omniDirection = ComputeVectorAndLength (ObjectLocalLightPosition[i],input.Position);
		
		// omnilight direction to texture space
		omniDirectionTextureSpace[i] = TextureSpaceTransform (omniDirection.xyz,input.BumpTangent,input.BumpNormal,binormal);
		omniDirectionTextureSpace[i] *= ComputeAttenuation (omniDirection.w,LightAttenuationFarStart[i],LightAttenuationFarEnd[i],LightAttenuationDelta[i]);
	}
	
	// omnilight 1 output to diffuse
	output.TextureSpaceLightVector1.xyz = omniDirectionTextureSpace[0] * 0.5f + 0.5f;
	
	// omnilight 2 output to diffuse
	output.TextureSpaceLightVector2.xyz = omniDirectionTextureSpace[1] * 0.5f + 0.5f;
	
	// Diffuse Uv output
	output.Diffuse1Uv = input.Diffuse1Uv;
	output.Diffuse2Uv = input.Diffuse2Uv;
		
	// Normal Uv output
	output.NormalUv = input.Diffuse1Uv;
		
	// Fog computation
	FOG_OPTION_COMPUTE(output, output.Position);		
		
	return output;
}

//------------------------------------------------------------------------------------------------------

CVertexShaderOutputSpec	OpaBumpLitGeomTech1Pass2 (const CVertexShaderInput input)
{
	CVertexShaderOutputSpec output;
	
	// output position into world+view+projection space
	output.Position = mul (input.Position,WorldCameraProjection);

	// binormal computation based on input tangent and normal in local space
	float3 binormal = input.Normal;
	
	// Camera vertex direction
	float3 cameraDirection = ComputeVector (ObjectLocalCameraPosition,input.Position);
	
	// Light vertex direction
	float4 omniDirection1 = ComputeVectorAndLength (ObjectLocalLightPosition[0],input.Position);
	
	// half view vector computation
	float3 halfView1 = ComputeHalfViewVector (omniDirection1.xyz,cameraDirection);
	//float3 halfView1 = reflect (-omniDirection1.xyz,cameraDirection);
		
	// omni direction to texture space
	float3 omniDirectionTextureSpace = TextureSpaceTransform (omniDirection1.xyz,input.BumpTangent,input.BumpNormal,binormal);
	
	// RICO bug de calcul de spec
	// ce vecteur de spec doit avoir les composantes entre -1 et 1 et non entre 0 et 1
	// car le calcul de spec est fait par l'instruction texm3x2pad 
	//output.TextureSpaceLightVector = omniDirectionTextureSpace * 0.5f + 0.5f;
	output.TextureSpaceLightVector = omniDirectionTextureSpace;
			
	// half view vector to texture space
	float3 halfView1TextureSpace = TextureSpaceTransform (halfView1,input.BumpTangent,input.BumpNormal,binormal);
	
	// RICO bug de calcul de spec
	// ce vecteur de spec doit avoir les composantes entre -1 et 1 et non entre 0 et 1
	// car le calcul de spec est fait par l'instruction texm3x2tex 
	//output.TextureSpaceLightVectorHalf = halfView1TextureSpace * 0.5f + 0.5f;
	output.TextureSpaceLightVectorHalf = halfView1TextureSpace;
	
	// Diffuse Uv output
	output.SpecularUv = input.Diffuse1Uv;
	
	// Normal Uv output
	output.NormalUv = input.Diffuse1Uv;
	
	output.SpecularStrength = dot (halfView1TextureSpace,binormal);
	output.SpecularStrength *= ComputeAttenuation (omniDirection1.w,LightAttenuationFarStart[0],LightAttenuationFarEnd[0],LightAttenuationDelta[0]);

	// fog computation
	FOG_OPTION_ON(output.SpecularStrength.w = fog_linear(output.Position);)

	return output;
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderOutputTech2	OpaBumpLitGeomTech2 (const CVertexShaderInput input)
{
	CVertexShaderOutputTech2 output;
	
	// output position into world+view+projection space
	output.Position = mul (input.Position,WorldCameraProjection);

	// normal computation based on input tangent and normal in local space
	float3 normal = input.Normal;
	
	float3 computedDiffuseColors[2];
	
	for (int i=0;i<2;i++)
	{
		// omnilight direction computation
		float4 omniDirection = ComputeVectorAndLength (ObjectLocalLightPosition[i],input.Position);
		
		// omnilight color computation
		float intensity = dot (omniDirection.xyz,normal);
		intensity = max(intensity,0.0f);
		intensity = min(intensity,1.0f);
		intensity *= ComputeAttenuation (omniDirection.w,LightAttenuationFarStart[i],LightAttenuationFarEnd[i],LightAttenuationDelta[i]);
		computedDiffuseColors[i].xyz = intensity * DiffuseColor[i];	
	}
	
	// Diffuse Uv output
	output.Diffuse1Uv = input.Diffuse1Uv;
	output.Diffuse2Uv = input.Diffuse2Uv;
		
	// Diffuse color output
	output.DiffuseColor = computedDiffuseColors[0] + computedDiffuseColors[1];

	// Fog computation
	FOG_OPTION_COMPUTE(output, output.Position);		
			
	return output;
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

sampler Diffuse1Sampler = sampler_state
{
	Texture		= <Diffuse1>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
    AddressV	= WRAP;
};

//------------------------------------------------------------------------------------------------------

sampler Diffuse2Sampler = sampler_state
{
	Texture		= <Diffuse2>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
    AddressV	= WRAP;
};

//------------------------------------------------------------------------------------------------------

sampler NormalMapSampler = sampler_state
{
	Texture		= <NormalMap>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
    AddressV	= WRAP;
};

//------------------------------------------------------------------------------------------------------

sampler GlossinessMapSampler = sampler_state
{
	Texture		= <GlossinessMap>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= CLAMP;
    AddressV	= CLAMP;
};

//------------------------------------------------------------------------------------------------------

sampler SpecularMapSampler = sampler_state
{
	Texture		= <SpecularMap>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= LINEAR;
	AddressU	= WRAP;
    AddressV	= WRAP;
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#define Pass1RenderStateBlock \
CullMode			= CW; \
ZEnable				= true; \
ZFunc				= LESSEQUAL; \
ZWriteEnable		= true; \
AlphaBlendEnable	= false; \
AlphaTestEnable		= true; \
AlphaRef			= 192; \
AlphaFunc			= GREATEREQUAL ; \
FOG_OPTION_PARAMETERS

//------------------------------------------------------------------------------------------------------

#define Pass2RenderStateBlock \
CullMode			= CW; \
ZEnable				= true; \
ZFunc				= LESSEQUAL; \
ZWriteEnable		= false; \
AlphaBlendEnable	= true; \
SrcBlend			= ONE; \
DestBlend			= ONE; \
AlphaTestEnable		= true; \
AlphaRef			= 192; \
AlphaFunc			= GREATEREQUAL; \
FogEnable			= false \

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	tech1
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		Sampler[0]	= <Diffuse1Sampler>;
		Sampler[1]	= <Diffuse2Sampler>;
		Sampler[2]	= <NormalMapSampler>;

		Pass1RenderStateBlock;
		
		VertexShader = compile vs_1_1 OpaBumpLitGeomTech1Pass1();
		
		PixelShaderConstant[0]		= <AmbientColor>;
		PixelShaderConstant[1]		= <DiffuseColor[0]>;
		PixelShaderConstant[3]		= <DiffuseColor[1]>;
		
		PixelShader = 
		asm
		{
			ps_1_1
			
			tex			t0	// Diffuse 1
			tex			t1	// Diffuse 2
			tex			t2	// Normal map
			
			dp3_sat r0.rgb,t2_bx2,v0_bx2 +mov_sat r0.a,t0.a // r1 = dp3 (light1 * normal)
			dp3_sat r1.rgb,t2_bx2,v1_bx2 +mov_sat r1.a,t0.a // r1 = dp3 (light1 * normal)
			mul r0.rgb,r0,c1
			mad r0.rgb,r1,c3,r0
			
			lrp		r1.rgb,t1.a,t1,t0
			
			mul_x2_sat r0.rgb,r0,r1
			//mad_sat r0.rgb,t0,c0,r0
			mad_sat r0.rgb,r1,c0,r0
		};
	}

	pass pass2
	{
		Sampler[0]	= <SpecularMapSampler>;
		Sampler[1]	= <NormalMapSampler>;
		Sampler[3]	= <GlossinessMapSampler>;
	
		Pass2RenderStateBlock;
		
		VertexShader = compile vs_1_1 OpaBumpLitGeomTech1Pass2();
		
		PixelShaderConstant[2]		= <SpecularColor>;
		
		PixelShader =
		#ifdef FOG_OPTION_ENABLE 
		asm
		{
			ps_1_1
			
			tex			t0	// Specular map
			tex			t1	// Normal map
			texm3x2pad	t2, t1_bx2 // Light vector * normal vector
			texm3x2tex	t3, t1_bx2 // Half vector * normal vector
			
			mov r0.rgb,t3 + mov r0.a,t0.a
			mul r0.rgb,r0,v0
			mul r0.rgb,r0,t0
			mul_x4 r0.rgb,r0,c2
			
			// fog factor stored in v0.a
			mul	r0.rgb, r0, v0.a
		};
		#else
		asm
		{
			ps_1_1
			
			tex			t0	// Specular map
			tex			t1	// Normal map
			texm3x2pad	t2, t1_bx2 // Light vector * normal vector
			texm3x2tex	t3, t1_bx2 // Half vector * normal vector
			
			mov r0.rgb,t3 + mov r0.a,t0.a
			mul r0.rgb,r0,v0
			mul r0.rgb,r0,t0
			mul_x4 r0.rgb,r0,c2
		};
		#endif
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	tech2
<
	int Priority = 1;
	int TechniqueIndex = 1;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		Sampler[0]	= <Diffuse1Sampler>;
		Sampler[1]	= <Diffuse2Sampler>;
		Sampler[2]	= <NormalMapSampler>;
		
		Pass1RenderStateBlock;
		
		VertexShader = compile vs_1_1 OpaBumpLitGeomTech1Pass1();
		
		PixelShaderConstant[0]		= <AmbientColor>;
		PixelShaderConstant[1]		= <DiffuseColor[0]>;
		PixelShaderConstant[3]		= <DiffuseColor[1]>;
		
		PixelShader = 
		asm
		{
			ps_1_1
			
			tex			t0	// Diffuse 1
			tex			t1	// Diffuse 2
			tex			t2	// Normal map
			
			dp3_sat r0.rgb,t2_bx2,v0_bx2 +mov_sat r0.a,t0.a // r1 = dp3 (light1 * normal)
			dp3_sat r1.rgb,t2_bx2,v1_bx2 +mov_sat r1.a,t0.a // r1 = dp3 (light1 * normal)
			mul r0.rgb,r0,c1
			mad r0.rgb,r1,c3,r0
			
			lrp		r1.rgb,t1.a,t1,t0
			
			mul_x2_sat r0.rgb,r0,r1
			mad_sat r0.rgb,r1,c0,r0
		};
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	tech3
<
	int Priority = 1;
	int TechniqueIndex = 2;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		Sampler[0]	= <Diffuse1Sampler>;
		Sampler[1]	= <Diffuse2Sampler>;
		
		Pass1RenderStateBlock;
		
		VertexShader = compile vs_1_1 OpaBumpLitGeomTech2();
		
		PixelShaderConstant[0]		= <AmbientColor>;
		
		PixelShader = 
		asm
		{
			ps_1_1
			
			tex			t0	// Diffuse 1
			tex			t1	// Diffuse 2
			
			lrp		r1.rgb,t1.a,t1,t0
			
			mul_x2_sat r0.rgb,v0,r1 +mov r0.a,t0.a
			mad_sat r0.rgb,r1,c0,r0
		};
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	techTnL_0
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = TNL_ONLY;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Standard";
>
{
	pass pass1
	{
		WorldTransform[0]		= <WorldCameraProjection>;
		IndexedVertexBlendEnable = false;
		
		LightType[0]			= POINT;
		LightAttenuation0[0]	= <LightAttenuationFarStart[0]>;
		LightAttenuation1[0]	= 0.0f;
		LightAttenuation2[0]	= <LightAttenuationFarEnd[0]>;
		LightAmbient[0]			= <AmbientColor>;
		LightDiffuse[0]			= <DiffuseColor[0]>;
		LightPosition[0]		= <ObjectLocalLightPosition[0]>;
		LightRange[0]			= <LightAttenuationDelta[0]>;
		LightEnable[0]			= true;
		LightEnable[1]			= false;
		LightEnable[2]			= false;
		LightEnable[3]			= false;
		LightEnable[4]			= false;
		LightEnable[5]			= false;
		LightEnable[6]			= false;
		LightEnable[7]			= false;
		Lighting				= true;
		MaterialAmbient			= {1.0f,1.0f,1.0f,1.0f};
		MaterialDiffuse			= {1.0f,1.0f,1.0f,1.0f};
		MaterialEmissive		= {0.0f,0.0f,0.0f,0.0f};
		MaterialPower			= 0.0f;
		MaterialSpecular		= {0.0f,0.0f,0.0f,0.0f};
		DiffuseMaterialSource	= COLOR1;
		AmbientMaterialSource	= MATERIAL;
		
		Sampler[0]	= <Diffuse1Sampler>;
		
		Pass1RenderStateBlock;
		
		ColorArg1[0]		= TEXTURE;
		ColorArg2[0]		= DIFFUSE;
		ColorOp[0]			= MODULATE2X;
		AlphaArg1[0]		= TEXTURE;
		AlphaOp[0]			= SELECTARG1;
		ColorOp[1]			= DISABLE;
		AlphaOp[1]			= DISABLE;
		
		VertexShader = NULL;
		
		PixelShader = NULL;
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

#include "mesh_shadow.fx"
#include "mesh_shadow_projector.fx"

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
