//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

//#define SHOW_PROJECTOR_UV

//------------------------------------------------------------------------------------------------------
//- Static parameters
//------------------------------------------------------------------------------------------------------

float rProjectorZClipNear = 0.5f;
float rProjectorZClipFadeStart = 1.0f;
float rProjectorZClipFadeEnd   = 1.3f;
float rProjectorZClipFadeSpan  = 0.3f;

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

struct	CVertexShaderProjectorInput
{
	float4	Position	:	POSITION;
	float3	Normal		:	NORMAL;
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderProjectorOutput
{
	float4	Position		:	POSITION;
	float4	ProjectorUv		:	TEXCOORD0;
	float4	ProjectorClip	:	TEXCOORD1; // used to texkill backfacing shadows
	float4  FadeLerpFactor	:	COLOR0;
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderProjectorOutput	ProjectorPass (const CVertexShaderProjectorInput input);

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderProjectorOutput	ProjectorPass (const CVertexShaderProjectorInput input)
{
	CVertexShaderProjectorOutput output;
	
	output.Position = mul (input.Position,WorldCameraProjection); // transform vertex to clip space
	
	float4 worldPosition = mul (input.Position,World);			  // transform vertex to world space
	output.ProjectorUv	 = mul (worldPosition,ProjectorMatrix);	  // transform from world space to projector clip space	
	output.ProjectorClip.w = output.ProjectorUv.w;
	//output.ProjectorUv.w = max (output.ProjectorUv.w,0.0f);		  // clamp w to 0.0
	
	output.ProjectorUv.xyz /= output.ProjectorUv.w;				  // homogeneous perspective divide
	output.ProjectorUv.w = 1.0f;

// ### Be bourrinadin
	float4 WorldNormal = mul(input.Normal, World);
	
	// texkill pixels with a Z depth in projector clip space less than near.
	output.ProjectorClip.x = output.ProjectorUv.z - rProjectorZClipNear;	// kill texels closer than near clip
	output.ProjectorClip.y = rProjectorZClipFadeEnd - output.ProjectorUv.z;		// kill texels farther than far clip
	output.ProjectorClip.z = dot(WorldNormal, (ProjectorPosition-worldPosition)); // kill texels backfacing projector
	output.ProjectorClip.w = 0.0f;

	float rLerpFactor = output.ProjectorUv.z - rProjectorZClipFadeStart;
	rLerpFactor /= rProjectorZClipFadeSpan;
	output.FadeLerpFactor = float4(rLerpFactor,rLerpFactor,rLerpFactor,rLerpFactor);
// ### Be bourrinadin
	
	return output;
}


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
/*
CVertexShaderProjectorOutputnVidia	ProjectorPassnVidia (const CVertexShaderProjectorInput input)
{
	CVertexShaderProjectorOutputnVidia output;
	
	output.Position = mul (input.Position,WorldCameraProjection);
	
	float4 worldPosition = mul (input.Position,World);
	output.ProjectorUv = mul (worldPosition,ProjectorMatrix);
	output.ProjectorUv.w = max (output.ProjectorUv.w,0.0f);
	
	output.ProjectorUv2.xy = output.ProjectorUv.xy;
	output.ProjectorUv2.xy /= output.ProjectorUv.w;
	
	output.ProjectorUv.xyz /= output.ProjectorUv.w;
	output.ProjectorUv.w = 1.0f;
	
	float3 worldNormal = mul (input.Normal,World);
	
	float3 cameraDirection = ComputeVector (ProjectorPosition.xyz,worldPosition.xyz);
	float angle = dot (cameraDirection,worldNormal);

	output.ProjectorColor.xyzw = step (0.1f,angle);
		
	return output;
}
*/

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

sampler ColorSampler = sampler_state
{
	Texture		= <ProjectorTextureColor>;
	MinFilter	= LINEAR;
	MagFilter	= LINEAR;
	MipFilter	= NONE;
	AddressU	= CLAMP;
    AddressV	= CLAMP;
};

/*
struct PS20_Output
{
	float4	Color	:	COLOR;
};

//------------------------------------------------------------------------------------------------------

PS20_Output	ProjectorPassPS20PixelShader	(const CVertexShaderProjectorOutputPS20 input)
{
	PS20_Output output;
	
	float4 texValZ = tex2D (ZSampler,input.ProjectorUv);
	float4 texValColor = tex2D (ColorSampler,input.ProjectorUv);
	
	float biasedZ = input.ProjectorUv.z;
	biasedZ = min (biasedZ,1.0f);
	float ZDiff = texValZ.r - biasedZ;
	float ShadowResult;

	ShadowResult = step (0.0f,ZDiff);
	
	output.Color.xyzw = ShadowResult + texValColor.g;
	output.Color.xyzw = min (1.0f,output.Color.x);
	
	output.Color.xyzw = ProjectorColorLow + output.Color.x * ProjectorColorHigh;
	
	return output;
}
*/

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	tech_projector
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Projection";
>
{
	pass pass1
	{
		CullMode			= CW;
		ZEnable				= true;
		ZFunc				= LESSEQUAL;
		ZWriteEnable		= false;
		AlphaBlendEnable	= true;
		//AlphaBlendEnable	= false;
		SrcBlend			= DESTCOLOR;
		DestBlend			= ZERO;
		//SrcBlend			= ONE;
		//DestBlend			= ONE;
		AlphaTestEnable		= false;
		
		//AlphaTestEnable		= true;
		//AlphaRef			= 255;
		//AlphaFunc			= LESS;
		
		FogEnable			= false;
		
		Texture[0]			= <ProjectorTextureColor>;
		
		MinFilter[0]		= LINEAR;
		MagFilter[0]		= LINEAR;
		MipFilter[0]		= NONE;
		
		MinFilter[1]		= LINEAR;
		MagFilter[1]		= LINEAR;
		MipFilter[1]		= NONE;
		
        AddressU[0]			= CLAMP;
        AddressV[0]			= CLAMP;
        
		VertexShader = compile vs_1_1 ProjectorPass();
		
		PixelShaderConstant[0]		= {0.5f,0.5f,0.5f,0.5f};
		PixelShaderConstant[1]		= {1.0f,1.0f,0.0f,0.0f};
		PixelShaderConstant[2]		= {1.0f,1.0f,1.0f,1.0f};
		PixelShaderConstant[3]		= <ProjectorColorLow>;
		PixelShaderConstant[4]		= <ProjectorColorHigh>;
		
		PixelShader = 
		
#ifdef SHOW_PROJECTOR_UV		
		asm
		{
			ps_1_1

			texcoord t0
			texkill t1 // reject bad texels (cf vertex shader)
			mul r0.rgb, t0, c1 + mov r0.a, c2.a
		};
#else
		asm
		{
			ps_1_1

			tex t0
			texkill t1 // reject bad texels (cf vertex shader)
			
			mad r0.rgb,t0,c4,c3 + mov r0.a, t0.a
			lrp r0, v0, c2, r0
		};
#endif
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------