//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------
//- Static parameters
//------------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

struct	CVertexShaderProjectorInput
{
	float4	Position	:	POSITION;
	float3	Weight		:	BLENDWEIGHT;
	float4	BoneIndex	:	BLENDINDICES;
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderProjectorOutputPS20
{
	float4	Position		:	POSITION;
	float4	ProjectorUv		:	TEXCOORD0;
	float4	ProjectorUv2	:	TEXCOORD1;
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderProjectorOutputnVidia
{
	float4	Position	:	POSITION;
	float4	ProjectorUv	:	TEXCOORD0;
	float4	ProjectorUv2	:	TEXCOORD1;
};

//------------------------------------------------------------------------------------------------------

struct	CVertexShaderProjectorOutputATI
{
	float4	Position		:	POSITION;
	float4	ProjectorUv		:	TEXCOORD0;
	float4	ProjectorUvZ	:	TEXCOORD1;
};

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderProjectorOutputPS20	ProjectorPassPS20	(const CVertexShaderProjectorInput input);
CVertexShaderProjectorOutputnVidia	ProjectorPassnVidia	(const CVertexShaderProjectorInput input);
CVertexShaderProjectorOutputATI	ProjectorPassATI		(const CVertexShaderProjectorInput input);

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderProjectorOutputPS20	ProjectorPassPS20 (const CVertexShaderProjectorInput input)
{
	CVertexShaderProjectorOutputPS20	output;
	CSkinningOutput					skinOutput;
	
	float3 fakeNormal = (0.0f,0.0f,0.0f);
	float3 fakeBumpTangent = (0.0f,0.0f,0.0f);
	float3 fakeBumpNormal = (0.0f,0.0f,0.0f);
	
	skinOutput =  ComputeSkinning4Bones	(input.Position, fakeNormal, fakeBumpTangent, fakeBumpNormal, 
										 input.BoneIndex,input.Weight,BoneArray);
	
	output.Position = mul (skinOutput.SkinnedPosition,CameraProjection);
	
	output.ProjectorUv = mul (skinOutput.SkinnedPosition,ProjectorMatrix);
	output.ProjectorUv.w = max (output.ProjectorUv.w,0.0f);
	
	output.ProjectorUv2 = output.ProjectorUv;
	
	return output;
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderProjectorOutputATI	ProjectorPassATI (const CVertexShaderProjectorInput input)
{
	CVertexShaderProjectorOutputATI	output;
	CSkinningOutput					skinOutput;
	
	float3 fakeNormal = (0.0f,0.0f,0.0f);
	float3 fakeBumpTangent = (0.0f,0.0f,0.0f);
	float3 fakeBumpNormal = (0.0f,0.0f,0.0f);
	
	skinOutput =  ComputeSkinning4Bones	(input.Position, fakeNormal, fakeBumpTangent, fakeBumpNormal, 
										 input.BoneIndex,input.Weight,BoneArray);
	
	output.Position = mul (skinOutput.SkinnedPosition,CameraProjection);
	
	output.ProjectorUv = mul (skinOutput.SkinnedPosition,ProjectorMatrix);
	
	float4 tempUv = mul (skinOutput.SkinnedPosition,ProjectorMatrixZBuffer);
	output.ProjectorUvZ.xyz = tempUv.z;
	output.ProjectorUvZ.w = tempUv.w;
	
	return output;
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

CVertexShaderProjectorOutputnVidia	ProjectorPassnVidia (const CVertexShaderProjectorInput input)
{
	CVertexShaderProjectorOutputnVidia	output;
	CSkinningOutput					skinOutput;
	
	float3 fakeNormal = (0.0f,0.0f,0.0f);
	float3 fakeBumpTangent = (0.0f,0.0f,0.0f);
	float3 fakeBumpNormal = (0.0f,0.0f,0.0f);
	
	skinOutput =  ComputeSkinning4Bones	(input.Position, fakeNormal, fakeBumpTangent, fakeBumpNormal, 
										 input.BoneIndex,input.Weight,BoneArray);
	
	output.Position = mul (skinOutput.SkinnedPosition,CameraProjection);
	
	output.ProjectorUv = mul (skinOutput.SkinnedPosition,ProjectorMatrix);
	output.ProjectorUv.w = max (output.ProjectorUv.w,0.0f);
	
	output.ProjectorUv2 = output.ProjectorUv;
	
	return output;
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	tech_projector_PS20
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Projection";
>
{
	pass pass1
	{
		CullMode			= CW;
		ZEnable				= true;
		ZFunc				= LESSEQUAL;
		ZWriteEnable		= false;
		AlphaBlendEnable	= true;
		SrcBlend			= DESTCOLOR;
		DestBlend			= ZERO;
		AlphaTestEnable		= false;
		
		
		Texture[0]			= <ProjectorTextureColor>;
		
		MinFilter[0]		= LINEAR;
		MagFilter[0]		= LINEAR;
		MipFilter[0]		= NONE;
		
        AddressU[0]			= CLAMP;
        AddressV[0]			= CLAMP;
        
		VertexShader = compile vs_1_1 ProjectorPassPS20();
		
		PixelShaderConstant[0]		= {0.5f,1.0f,-0.001f,-0.08f};
		
		PixelShader = 
		asm
		{
			ps_2_0
			
			dcl		t0.xyzw
			dcl		t1.xyzw
			dcl_2d	s0
			
			texldp	r0,t0,s0
		
			rcp		r1.w,t1.w
			
			mul		r1.xyzw,t1.z,r1.w
			add		r1.xyzw,r1,c0.w
						
			sub		r0.xyzw,r1.x,r0.x
			cmp		r0.xyzw,-r0.x,c0.y,c0.x
			
			mov		oC0,r0
		};
	}
}


//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	tech_projector_ATI
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Projection";
>
{
	pass pass1
	{
		CullMode			= CW;
		ZEnable				= true;
		ZFunc				= LESSEQUAL;
		ZWriteEnable		= false;
		AlphaBlendEnable	= true;
		//AlphaBlendEnable	= false;
		SrcBlend			= DESTCOLOR;
		DestBlend			= ZERO;
		AlphaTestEnable		= false;
		
		Texture[0]			= <ProjectorTextureColor>;
		MinFilter[0]		= POINT;
		MagFilter[0]		= POINT;
		//MinFilter[0]		= LINEAR;
		//MagFilter[0]		= LINEAR;
        AddressU[0]			= CLAMP;
        AddressV[0]			= CLAMP;
        
        Texture[1]			= <ShadowZTable>;
		MinFilter[1]		= POINT;
		MagFilter[1]		= POINT;
        AddressU[1]			= CLAMP;
        AddressV[1]			= CLAMP;
        
		VertexShader = compile vs_1_1 ProjectorPassATI();

		PixelShaderConstant[0]		= {0.0f,0.0f,0.0f,0.0f};
		PixelShaderConstant[1]		= {1.0f,1.0f,1.0f,1.0f};
		PixelShaderConstant[2]		= {0.25f,0.25f,0.25f,0.25f};
		PixelShaderConstant[3]		= {-0.08f,-0.08f,-0.08f,-0.08f};
		PixelShaderConstant[4]		= {0.5f,0.5f,0.5f,0.5f};
		
		PixelShader = 
		asm
		{
			ps_1_4

			texld	r0,t0_dw.xyw
			texld	r1,t1_dw.xyw
			
			mov_d8 r0.r,r0.r
			add r0.r,r0.r,r0.g
			
			mov_d8 r1.r,r1.r
			add r1.r,r1.r,r1.g
			add r1.r,r1.r,c3.r
			
			sub r0.r,r1.r,r0.r
			cmp r0.rgba,-r0.r,c1.r,c0.r
			
			add r0,r0,c4
		};
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------

technique	tech_projector_nVidia
<
	int Priority = 1;
	int TechniqueIndex = 0;
	int DeviceType = HWSHADER_ONLY;
	int LightingType = INTEGRATED_LIGHTING;
	string RenderingType = "Projection";
>
{
	pass pass1
	{
		CullMode			= CW;
		ZEnable				= true;
		ZFunc				= LESSEQUAL;
		ZWriteEnable		= false;
		AlphaBlendEnable	= true;
		//AlphaBlendEnable	= false;
		SrcBlend			= DESTCOLOR;
		DestBlend			= ZERO;
		AlphaTestEnable		= false;
		
		
		Texture[0]			= <ProjectorTextureDepth>;
		Texture[1]			= <ProjectorTextureColor>;
		
		MinFilter[0]		= LINEAR;
		MagFilter[0]		= LINEAR;
		MipFilter[0]		= LINEAR;
		
		MinFilter[1]		= LINEAR;
		MagFilter[1]		= LINEAR;
		MipFilter[1]		= LINEAR;
		
        AddressU[0]			= CLAMP;
        AddressV[0]			= CLAMP;
        
        AddressU[1]			= CLAMP;
        AddressV[1]			= CLAMP;
		
		VertexShader = compile vs_1_1 ProjectorPassnVidia();

		PixelShaderConstant[0]		= {0.5f,0.5f,0.5f,0.5f};
		
		PixelShader = 
		asm
		{
			ps_1_1
			
			tex		t0
			tex		t1
						
			mov r0,t0
			
			add		r0,r0,1-t1
			add		r0,r0,c0
		};
	}
}

//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------